/* chd.c  --  char/hex dump 0.2,  like debug 
 * (C) 1998 David Lee Lambert, GPL
 * plan:  make it imitate od as much as possible
 */

#include <stdio.h>

char * version = "chd 0.2 (C) David Lee Lambert (GPL)";

char *helpstrings[] = {
"chd:  char/hex dump",
"",
"  Syntax:  chd [ -h | -v ] [<file>] [-d] [-a]",  
"Dumps a literal hex form of a file or the standard input.",
"Options: ",
"  -h   This help",
"  -v   Version",
"  -d   Use decimal line numbers",
"  -a   Print escape sequences for most unprintable chars,  and",
"       display all chars in the range 128--254 (this may cause",
"       lines to be longer than 80 characters)",
"Options may be combined,  like chd -ad", NULL};

int main(int argc, char *argv[])
{
   unsigned char buf[16], printable_buf[17];
   int len, i, j;
   long offset = 0;
   FILE *in;
   int showver = 0, showhelp = 0, usedecimal = 0, useallchar = 0,
      foundfile = 0;

   /* parse options */
   in = stdin;
   for (i=1;i<argc;i++) {
      if (argv[i][0] == '-') {
	 for (j=1;argv[i][j];j++) {
	    if (argv[i][j] == 'v') {
	       showver = 1;
	    } else if (argv[i][j] == 'h') {
	       showhelp = 1;
	    } else if (argv[i][j] == 'd') {
	       usedecimal = 1;
	    } else if (argv[i][j] == 'a') {
	       useallchar = 1;
	    } else {
	       fprintf(stderr, "chd: Bad argument: -%c\n", argv[i][j]);
	       showhelp = 1;
	    }
	 }
      } else if (argv[i][0] == '/' && argv[i][1] == '?') {
	 showhelp = 1; /* for MS-DOS */
      } else {
	 if (!foundfile) {
	    in = fopen(argv[i], "r");
	    if (!in) {
	       perror("chd");
	       showhelp = 1;
	    } else foundfile = 1;
	 } else {
	    fprintf(stderr, "chd: I don't do multiple files\n");
	 }
      }
   }

   if (showver) printf("%s\n", version);

   if (showhelp) {
      for (i=0;helpstrings[i];i++) printf("%s\n", helpstrings[i]);
      return 1;
   }

   if (showver && !foundfile) return 0;

   /* main loop */
   while (!feof(in)) {
      /* read buffer */
      len = fread(buf, sizeof(char), 16, in);
      if (len == 0) break;
      /* set up printable representation */
      if (!useallchar) {
	 for (i=0;i<len;i++) printable_buf[i] = 
				(buf[i] < ' ' || buf[i] >= 127)
				? '.' : buf[i];
      	 printable_buf[i++] = 0;
      }
      /* display it */
      printf(usedecimal ? "%5ld " : "%08lx  ", offset);
      for (i=0;i<len;i++) printf("%02X%c", buf[i], (i==7)?'-':' ');
      for (;i<16;i++) printf("   ");
      if (!useallchar) {
	 printf("  %s\n", printable_buf);
      } else {
	 for (i=0;i<len;i++) {
	    if (buf[i] == 0) {
	       printf("\\0");
	    } else if (buf[i] == '\t') {
	       printf("\\t");
	    } else if (buf[i] == '\n') {
	       printf("\\n");
	    } else if (buf[i] == '\a') {
	       printf("\\a");
	    } else if (buf[i] == '\r') {
	       printf("\\r");
	    } else if (buf[i] == 255 || buf[i] == 127 || buf[i] < 32) {
	       printf("\\%03o", buf[i]);
	    } else if (i>0 && buf[i-1]==0 && ((int)buf[i]-'0')<8) {
	       printf("00%c", buf[i]); /* \00010 -> \010 won't occur */
	    } else {
	       printf("%c", buf[i]);
	    }
	 }
	 printf("\n");
      }
		       
      /* finish the loop calculation */
      offset += len;
   };

   return 0;
}
